@@ -298,8 +298,9 @@ i830PllIsValid(xf86CrtcPtr crtc, intel_clock_t *clock)
 }
 
 /**
- * Returns a set of divisors for the desired target clock with the given refclk,
- * or FALSE.  Divisor values are the actual divisors for
+ * Returns a set of divisors for the desired target clock with the given
+ * refclk, or FALSE.  The returned values represent the clock equation:
+ * reflck * (5 * (m1 + 2) + (m2 + 2)) / (n + 2) / p1 / p2.
  */
 static Bool
 i830FindBestPLL(xf86CrtcPtr crtc, int target, int refclk, intel_clock_t *best_clock)
@@ -310,10 +311,23 @@ i830FindBestPLL(xf86CrtcPtr crtc, int target, int refclk, intel_clock_t *best_cl
     const intel_limit_t   *limit = intel_limit (crtc);
     int err = target;
 
-    if (target < limit->p2.dot_limit)
-	clock.p2 = limit->p2.p2_slow;
-    else
-	clock.p2 = limit->p2.p2_fast;
+    if (IS_I9XX(pI830) && i830PipeHasType(crtc, I830_OUTPUT_LVDS) &&
+	(INREG(LVDS) & LVDS_PORT_EN) != 0)
+    {
+	/* For LVDS, if the panel is on, just rely on its current settings for
+	 * dual-channel.  We haven't figured out how to reliably set up
+	 * different single/dual channel state, if we even can.
+	 */
+	if ((INREG(LVDS) & LVDS_CLKB_POWER_MASK) == LVDS_CLKB_POWER_UP)
+	    clock.p2 = limit->p2.p2_fast;
+	else
+	    clock.p2 = limit->p2.p2_slow;
+    } else {
+	if (target < limit->p2.dot_limit)
+	    clock.p2 = limit->p2.p2_slow;
+	else
+	    clock.p2 = limit->p2.p2_fast;
+    }
 
     memset (best_clock, 0, sizeof (*best_clock));
 
@@ -578,6 +592,8 @@ static void
 i830_crtc_commit (xf86CrtcPtr crtc)
 {
     crtc->funcs->dpms (crtc, DPMSModeOn);
+    if (crtc->scrn->pScreen != NULL)
+	xf86_reload_cursors (crtc->scrn->pScreen);
 }
 
 void
@@ -657,23 +673,23 @@ i830_get_core_clock_speed(ScrnInfoPtr pScrn)
  * or -1 if the panel fitter is not present or not in use
  */
 static int
-i830_panel_fitter_pipe (I830Ptr	pI830)
+i830_panel_fitter_pipe(I830Ptr pI830)
 {
     CARD32  pfit_control;
-    
+
     /* i830 doesn't have a panel fitter */
     if (IS_I830(pI830))
 	return -1;
-    
+
     pfit_control = INREG(PFIT_CONTROL);
-    
+
     /* See if the panel fitter is in use */
     if ((pfit_control & PFIT_ENABLE) == 0)
 	return -1;
-    
+
     /* 965 can place panel fitter on either pipe */
     if (IS_I965G(pI830))
-	return (pfit_control >> 29) & 0x3;
+	return (pfit_control & PFIT_PIPE_MASK) >> PFIT_PIPE_SHIFT;
 
     /* older chips can only use pipe 1 */
     return 1;
@@ -868,8 +884,17 @@ i830_crtc_mode_set(xf86CrtcPtr crtc, DisplayModePtr mode,
     if (i830_panel_fitter_pipe (pI830) == pipe)
 	OUTREG(PFIT_CONTROL, 0);
 
-    i830PrintPll("chosen", &clock);
-    ErrorF("clock regs: 0x%08x, 0x%08x\n", (int)dpll, (int)fp);
+    if (pI830->debug_modes) {
+	xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+		   "Mode for pipe %c:\n", pipe == 0 ? 'A' : 'B');
+	xf86PrintModeline(pScrn->scrnIndex, mode);
+	if (!xf86ModesEqual(mode, adjusted_mode)) {
+	    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+		       "Adjusted mode for pipe %c:\n", pipe == 0 ? 'A' : 'B');
+	    xf86PrintModeline(pScrn->scrnIndex, mode);
+	}
+	i830PrintPll("chosen", &clock);
+    }
 
     if (dpll & DPLL_VCO_ENABLE)
     {
@@ -879,13 +904,38 @@ i830_crtc_mode_set(xf86CrtcPtr crtc, DisplayModePtr mode,
 	usleep(150);
     }
 
+    /* The LVDS pin pair needs to be on before the DPLLs are enabled.
+     * This is an exception to the general rule that mode_set doesn't turn
+     * things on.
+     */
     if (is_lvds)
     {
-	/* The LVDS pin pair needs to be on before the DPLLs are enabled.
-	 * This is an exception to the general rule that mode_set doesn't turn
-	 * things on.
+	CARD32 lvds = INREG(LVDS);
+
+	lvds |= LVDS_PORT_EN | LVDS_A0A2_CLKA_POWER_UP | LVDS_PIPEB_SELECT;
+	/* Set the B0-B3 data pairs corresponding to whether we're going to
+	 * set the DPLLs for dual-channel mode or not.
 	 */
-	OUTREG(LVDS, INREG(LVDS) | LVDS_PORT_EN | LVDS_PIPEB_SELECT);
+	if (clock.p2 == 7)
+	    lvds |= LVDS_B0B3_POWER_UP | LVDS_CLKB_POWER_UP;
+	else
+	    lvds &= ~(LVDS_B0B3_POWER_UP | LVDS_CLKB_POWER_UP);
+
+	/* It would be nice to set 24 vs 18-bit mode (LVDS_A3_POWER_UP)
+	 * appropriately here, but we need to look more thoroughly into how
+	 * panels behave in the two modes.
+	 */
+
+	/* Enable dithering if we're in 18-bit mode. */
+	if (IS_I965G(pI830))
+	{
+	    if ((lvds & LVDS_A3_POWER_MASK) == LVDS_A3_POWER_UP)
+		lvds |= LVDS_DITHER_ENABLE;
+	    else
+		lvds &= ~LVDS_DITHER_ENABLE;
+	}
+
+	OUTREG(LVDS, lvds);
 	POSTING_READ(LVDS);
     }
 
@@ -1334,6 +1384,12 @@ static const xf86CrtcFuncsRec i830_crtc_funcs = {
     .shadow_create = i830_crtc_shadow_create,
     .shadow_allocate = i830_crtc_shadow_allocate,
     .shadow_destroy = i830_crtc_shadow_destroy,
+    .set_cursor_colors = i830_crtc_set_cursor_colors,
+    .set_cursor_position = i830_crtc_set_cursor_position,
+    .show_cursor = i830_crtc_show_cursor,
+    .hide_cursor = i830_crtc_hide_cursor,
+/*    .load_cursor_image = i830_crtc_load_cursor_image, */
+    .load_cursor_argb = i830_crtc_load_cursor_argb,
     .destroy = NULL, /* XXX */
 };
 
